/*
 * Converts plan9 image to PCL
 */
#include <u.h>
#include <libc.h>
#include <draw.h>
#include <memdraw.h>

#define ESC 0x1B

typedef void(sendfn)(uchar *, int, int);

enum {
	PIN24,
	PIN9,
};

static int pins = PIN24;
static int mval = 39;
static int rowh = 3;
static int debug = 0;
static int random = 0;

static int
img2dot(int i, int w)
{
	int x,y;
	x = i % w;
	y = i / w;
	return x * rowh * 8 + y;
}

static int
randbit(int on)
{
	if(!random)
		return on < 0x90;
	return frand() * 0xff > on;
}

static void
biton(uchar * data, int i)
{
	data[i / 8] |= 0x80 >> i % 8;
}

static void
sendline(uchar * data, int m, int sz, int w)
{
	int i;
	if(debug){
		print("%c*%d,%d,%d,", ESC, m, w % 256, w / 256);
		for(i = 0; i < sz; i++)
			print("%02x.", data[i]);
	}else{
		print("%c*%c%c%c", ESC, m, w % 256, w / 256);
		write(1, data, sz);
	}
	print("%c%c", 13, 10);
}

static void
sendimg(uchar *dat, int w, int h)
{
	int i, r, imgz, rowz;
	uchar *outstr;
	
	imgz = w * h;
	rowz = w * rowh;
	outstr = malloc(rowz);
	
	if(pins == PIN9)
		print("%c3%c", ESC, 24);
	else
		print("%c+%c", ESC, 48);
	for(r = 0; r * rowz * 8 < imgz; r++){
		for(i = 0; i < rowz; i++)
			outstr[i] = 0;
		for(i = 0; i < rowz * 8 && (i + r * rowz * 8 < imgz); i++){
			if(randbit(dat[i + r * rowz * 8]))
				biton(outstr, img2dot(i, w));
		}
		sendline(outstr, mval, rowz, w);
	}
	free(outstr);
}

static void
getimg(sendfn f)
{
	int w, h, n;
	uchar *buf;
	Memimage *o;
	
	if(memimageinit() < 0)
		sysfatal("memimageinit: %r");
	o = readmemimage(0);
	if(o == nil)
		sysfatal("readmemimage: %r");
	/* for now, only use the 88 channel. */
	switch(o->chan){
	case 88:
		break;
	default:
		print("Chan: %ld\n", o->chan);
		sysfatal("chan: %r");
	}

	w = Dx(o->r);
	h = Dy(o->r);
	n = 4 * w * h * sizeof(uchar);
	buf = malloc(n);
	if(buf == nil)
		sysfatal("malloc: %r");
	if(unloadmemimage(o, o->r, buf, n) < 0)
		sysfatal("unloadimage: %r");
	freememimage(o);
	f(buf, w, h);
	free(buf);
}

static void
usage(void)
{
	fprint(2, "usage: %s [-m mval]\n", argv0);
	exits("usage");
}

static void
setmval(int m)
{
	switch(m){
	case 0: case 1: case 2: case 3:
	case 4: case 5: case 6: case 7:
		pins = PIN9;
		mval = m;
		rowh = 1;
		break;
	case 64: case 65: case 70:
	case 71: case 72: case 73:
		rowh = 6;
	case 32: case 33: case 38: case 39: case 40:
		pins = PIN24;
		mval = m;
		break;
	default:
		print("Invalid m-value\n");
		exits("mval");
	}
}

void
main(int argc, char* argv[])
{
	srand(time(0));
	ARGBEGIN{
	case 'r':
		random = 1;
		break;
	case 'd':
		debug = 1;
		break;
	case 'm':
		setmval(atoi(ARGF()));
		break;
	default:
		usage();
		break;
	}ARGEND;

	getimg(sendimg);
	
	exits(nil);
}
